#!/usr/bin/env python
# coding=utf-8

import os
import sys, json, re
from workflow import Workflow, ICON_INFO, ICON_ERROR, web
from commands import getoutput
import socket
from urlparse import urlparse

NAME_TO_ISO_CODE = {
    u'阿富汗': u'AF',
    u'奥兰群岛': u'AX',
    u'阿尔巴尼亚': u'AL',
    u'阿尔及利亚': u'DZ',
    u'美属萨摩亚': u'AS',
    u'安道尔': u'AD',
    u'安哥拉': u'AO',
    u'安圭拉': u'AI',
    u'南极洲': u'AQ',
    u'安提瓜和巴布达': u'AG',
    u'阿根廷': u'AR',
    u'亚美尼亚': u'AM',
    u'阿鲁巴': u'AW',
    u'澳大利亚': u'AU',
    u'奥地利': u'AT',
    u'阿塞拜疆': u'AZ',
    u'巴哈马': u'BS',
    u'巴林': u'BH',
    u'孟加拉国': u'BD',
    u'巴巴多斯': u'BB',
    u'白俄罗斯': u'BY',
    u'比利时': u'BE',
    u'伯利兹': u'BZ',
    u'贝宁': u'BJ',
    u'百慕大': u'BM',
    u'不丹': u'BT',
    u'玻利维亚': u'BO',
    u'波黑': u'BA',
    u'博茨瓦纳': u'BW',
    u'布维岛': u'BV',
    u'巴西': u'BR',
    u'英属印度洋领地': u'IO',
    u'文莱': u'BN',
    u'保加利亚': u'BG',
    u'布基纳法索': u'BF',
    u'布隆迪': u'BI',
    u'柬埔寨': u'KH',
    u'喀麦隆': u'CM',
    u'加拿大': u'CA',
    u'佛得角': u'CV',
    u'开曼群岛': u'KY',
    u'中非': u'CF',
    u'乍得': u'TD',
    u'智利': u'CL',
    u'中国': u'CN',
    u'圣诞岛': u'CX',
    u'科科斯（基林）群岛': u'CC',
    u'哥伦比亚': u'CO',
    u'科摩罗': u'KM',
    u'刚果（布）': u'CG',
    u'刚果（金）': u'CD',
    u'库克群岛': u'CK',
    u'哥斯达黎加': u'CR',
    u'科特迪瓦': u'CI',
    u'克罗地亚': u'HR',
    u'古巴': u'CU',
    u'塞浦路斯': u'CY',
    u'捷克': u'CZ',
    u'丹麦': u'DK',
    u'吉布提': u'DJ',
    u'多米尼克': u'DM',
    u'多米尼加': u'DO',
    u'厄瓜多尔': u'EC',
    u'埃及': u'EG',
    u'萨尔瓦多': u'SV',
    u'赤道几内亚': u'GQ',
    u'厄立特里亚': u'ER',
    u'爱沙尼亚': u'EE',
    u'埃塞俄比亚': u'ET',
    u'福克兰群岛（马尔维纳斯）': u'FK',
    u'法罗群岛': u'FO',
    u'斐济': u'FJ',
    u'芬兰': u'FI',
    u'法国': u'FR',
    u'法属圭亚那': u'GF',
    u'法属波利尼西亚': u'PF',
    u'法属南部领地': u'TF',
    u'加蓬': u'GA',
    u'冈比亚': u'GM',
    u'格鲁吉亚': u'GE',
    u'德国': u'DE',
    u'加纳': u'GH',
    u'直布罗陀': u'GI',
    u'希腊': u'GR',
    u'格陵兰': u'GL',
    u'格林纳达': u'GD',
    u'瓜德罗普': u'GP',
    u'关岛': u'GU',
    u'危地马拉': u'GT',
    u'格恩西岛': u'GG',
    u'几内亚': u'GN',
    u'几内亚比绍': u'GW',
    u'圭亚那': u'GY',
    u'海地': u'HT',
    u'赫德岛和麦克唐纳岛': u'HM',
    u'梵蒂冈': u'VA',
    u'洪都拉斯': u'HN',
    u'香港': u'HK',
    u'匈牙利': u'HU',
    u'冰岛': u'IS',
    u'印度': u'IN',
    u'印度尼西亚': u'ID',
    u'伊朗': u'IR',
    u'伊拉克': u'IQ',
    u'爱尔兰': u'IE',
    u'英国属地曼岛': u'IM',
    u'以色列': u'IL',
    u'意大利': u'IT',
    u'牙买加': u'JM',
    u'日本': u'JP',
    u'泽西岛': u'JE',
    u'约旦': u'JO',
    u'哈萨克斯坦': u'KZ',
    u'肯尼亚': u'KE',
    u'基里巴斯': u'KI',
    u'朝鲜': u'KP',
    u'韩国': u'KR',
    u'科威特': u'KW',
    u'吉尔吉斯斯坦': u'KG',
    u'老挝': u'LA',
    u'拉脱维亚': u'LV',
    u'黎巴嫩': u'LB',
    u'莱索托': u'LS',
    u'利比里亚': u'LR',
    u'利比亚': u'LY',
    u'列支敦士登': u'LI',
    u'立陶宛': u'LT',
    u'卢森堡': u'LU',
    u'澳门': u'MO',
    u'前南马其顿': u'MK',
    u'马达加斯加': u'MG',
    u'马拉维': u'MW',
    u'马来西亚': u'MY',
    u'马尔代夫': u'MV',
    u'马里': u'ML',
    u'马耳他': u'MT',
    u'马绍尔群岛': u'MH',
    u'马提尼克': u'MQ',
    u'毛利塔尼亚': u'MR',
    u'毛里求斯': u'MU',
    u'马约特': u'YT',
    u'墨西哥': u'MX',
    u'密克罗尼西亚联邦': u'FM',
    u'摩尔多瓦': u'MD',
    u'摩纳哥': u'MC',
    u'蒙古': u'MN',
    u'黑山': u'ME',
    u'蒙特塞拉特': u'MS',
    u'摩洛哥': u'MA',
    u'莫桑比克': u'MZ',
    u'缅甸': u'MM',
    u'纳米比亚': u'NA',
    u'瑙鲁': u'NR',
    u'尼泊尔': u'NP',
    u'荷兰': u'NL',
    u'荷属安的列斯': u'AN',
    u'新喀里多尼亚': u'NC',
    u'新西兰': u'NZ',
    u'尼加拉瓜': u'NI',
    u'尼日尔': u'NE',
    u'尼日利亚': u'NG',
    u'纽埃': u'NU',
    u'诺福克岛': u'NF',
    u'北马里亚纳': u'MP',
    u'挪威': u'NO',
    u'阿曼': u'OM',
    u'巴基斯坦': u'PK',
    u'帕劳': u'PW',
    u'巴勒斯坦': u'PS',
    u'巴拿马': u'PA',
    u'巴布亚新几内亚': u'PG',
    u'巴拉圭': u'PY',
    u'秘鲁': u'PE',
    u'菲律宾': u'PH',
    u'皮特凯恩': u'PN',
    u'波兰': u'PL',
    u'葡萄牙': u'PT',
    u'波多黎各': u'PR',
    u'卡塔尔': u'QA',
    u'留尼汪': u'RE',
    u'罗马尼亚': u'RO',
    u'俄罗斯联邦': u'RU',
    u'卢旺达': u'RW',
    u'圣赫勒拿': u'SH',
    u'圣基茨和尼维斯': u'KN',
    u'圣卢西亚': u'LC',
    u'圣皮埃尔和密克隆': u'PM',
    u'圣文森特和格林纳丁斯': u'VC',
    u'萨摩亚': u'WS',
    u'圣马力诺': u'SM',
    u'圣多美和普林西比': u'ST',
    u'沙特阿拉伯': u'SA',
    u'塞内加尔': u'SN',
    u'塞尔维亚': u'RS',
    u'塞舌尔': u'SC',
    u'塞拉利昂': u'SL',
    u'新加坡': u'SG',
    u'斯洛伐克': u'SK',
    u'斯洛文尼亚': u'SI',
    u'所罗门群岛': u'SB',
    u'索马里': u'SO',
    u'南非': u'ZA',
    u'南乔治亚岛和南桑德韦奇岛': u'GS',
    u'西班牙': u'ES',
    u'斯里兰卡': u'LK',
    u'苏丹': u'SD',
    u'苏里南': u'SR',
    u'斯瓦尔巴岛和扬马延岛': u'SJ',
    u'斯威士兰': u'SZ',
    u'瑞典': u'SE',
    u'瑞士': u'CH',
    u'叙利亚': u'SY',
    u'台湾': u'TW',
    u'塔吉克斯坦': u'TJ',
    u'坦桑尼亚': u'TZ',
    u'泰国': u'TH',
    u'东帝汶': u'TL',
    u'多哥': u'TG',
    u'托克劳': u'TK',
    u'汤加': u'TO',
    u'特立尼达和多巴哥': u'TT',
    u'突尼斯': u'TN',
    u'土耳其': u'TR',
    u'土库曼斯坦': u'TM',
    u'特克斯和凯科斯群岛': u'TC',
    u'图瓦卢': u'TV',
    u'乌干达': u'UG',
    u'乌克兰': u'UA',
    u'阿联酋': u'AE',
    u'英国': u'GB',
    u'美国': u'US',
    u'美国本土外小岛屿': u'UM',
    u'乌拉圭': u'UY',
    u'乌兹别克斯坦': u'UZ',
    u'瓦努阿图': u'VU',
    u'委内瑞拉': u'VE',
    u'越南': u'VN',
    u'英属维尔京群岛': u'VG',
    u'美属维尔京群岛': u'VI',
    u'瓦利斯和富图纳': u'WF',
    u'西撒哈拉': u'EH',
    u'也门': u'YE',
    u'赞比亚': u'ZM',
    u'津巴布韦': u'ZW'
}

def flag(code):
    OFFSET = 127397
    country = code
    if country is None:
        return None
    points = [ord(x) + OFFSET for x in country.upper()]
    try:
        return chr(points[0]) + chr(points[1])
    except ValueError:
        return ('\\U%08x\\U%08x' % tuple(points)).decode('unicode-escape')

regex_ipv4 = '(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)'
regex_hostname = '(?:(?:[a-z]|[a-z][a-z0-9\-]*[a-z0-9])\.)+(?:[a-z]|[a-z][a-z0-9\-]*[a-z0-9])+'
regex_ipv6 = '(?:(?:[0-9a-f]{1,4}:){7}(?:[0-9a-f]{1,4}|:))|(?:(?:[0-9a-f]{1,4}:){6}(?::[0-9a-f]{1,4}|(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){5}(?:(?:(?::[0-9a-f]{1,4}){1,2})|:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(?:(?:[0-9a-f]{1,4}:){4}(?:(?:(?::[0-9a-f]{1,4}){1,3})|(?:(?::[0-9a-f]{1,4})?:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){3}(?:(?:(?::[0-9a-f]{1,4}){1,4})|(?:(?::[0-9a-f]{1,4}){0,2}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){2}(?:(?:(?::[0-9a-f]{1,4}){1,5})|(?:(?::[0-9a-f]{1,4}){0,3}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?:(?:[0-9a-f]{1,4}:){1}(?:(?:(?::[0-9a-f]{1,4}){1,6})|(?:(?::[0-9a-f]{1,4}){0,4}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(?::(?:(?:(?::[0-9a-f]{1,4}){1,7})|(?:(?::[0-9a-f]{1,4}){0,5}:(?:(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(?:\.(?:25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))'
regex_host = '((%s)|(%s)|(%s))' % (
    regex_ipv4, regex_ipv6, regex_hostname)
re_host = re.compile(regex_host, re.IGNORECASE)


def extract_host(input_text):
    m = re_host.search(input_text)
    if m:
        return m.group()
    else:
        return None

def get_location_information(ip):

    LOCATION_QUERY_URL = 'http://www.ip138.com/ips138.asp'
    FEATURE_BEGIN_STR = '<td align="center"><ul class="ul1"><li>'
    FEATURE_END_STR = '</li></ul></td>'
    FEATURE_SPLIT_STR = '</li><li>'

    try:
        rt = web.get(LOCATION_QUERY_URL, dict(ip=ip, action=2))
        rt.raise_for_status()
        rts = rt.text[
            rt.text.find(FEATURE_BEGIN_STR) + len(FEATURE_BEGIN_STR) :
            rt.text.find(FEATURE_END_STR)]
        rtlist = rts.split(FEATURE_SPLIT_STR)

        # 去掉前缀和多余空格，最长的即是最优解
        result = ''
        for val in rtlist:
            rl = val[val.find(u'：') + 1:].strip().split()
            # 对诸如：北京市北京市 这种字符串修整为：北京市
            for i, k in enumerate(rl):
                size = len(k)
                if size % 2 == 0:
                    size /= 2
                    lhalf = k[:size]
                    rhalf = k[size:]
                    if lhalf == rhalf:
                        rl[i] = lhalf

            temp = ' '.join(rl)
            if len(temp) > len(result):
                result = temp
    except Exception:
        result = None

    return result

def get_local_ip():
    return getoutput('ipconfig getifaddr en0')

def ipip(ip):
    API = 'http://freeapi.ipip.net/%s' % ip
    try:
        rt = web.get(API)
        rt.raise_for_status()
        response = rt.text
        response.decode(unicode) #unicode(response)
        print response
        if response:
            # response = ast.literal_eval(response)
            # response = [n.strip() for n in response]
            # response = map(unicode,response)
            response = json.loads(response)
            pass
        else:
            return
    except Exception:
        ip = None
    response = json.loads(response)
    ret = {}
    if response[1] == response[2]:
        ret["result"] = response[0]+" "+response[1]+" "+response[3]+response[4]
    else:
        ret["result"] = response[0]+" "+response[1]+response[2]+" "+response[3]+" "+response[4]
    try:
        if response[0] == u'中国':
            if response[1] == u'香港' or response[1] == u'澳门' or response[1] == u'台湾':
                ret["emoji"] = flag(NAME_TO_ISO_CODE[response[1]].lower())
                ret["flag"] = 'flags/'+NAME_TO_ISO_CODE[response[1]].lower()+'.png'
            else:
                ret["emoji"] = flag(NAME_TO_ISO_CODE[response[0]].lower())
                ret["flag"] = 'flags/'+NAME_TO_ISO_CODE[response[0]].lower()+'.png'
        else:
            ret["emoji"] = flag(NAME_TO_ISO_CODE[response[0]].lower())
            ret["flag"] = 'flags/'+NAME_TO_ISO_CODE[response[0]].lower()+'.png'
    except KeyError:
        ret["emoji"] = u'\U0001F30E'
        ret["flag"] = 'flags/globe.png'
    return ret

def get_public_ip():
    PUBLIC_IP_QUERY_URL = 'http://1212.ip138.com/ic.asp'
    try:
        rt = web.get(PUBLIC_IP_QUERY_URL)
        rt.raise_for_status()
        ip = rt.text[rt.text.find('[') + 1 : rt.text.find(']')]
    except Exception:
        ip = None

    return ip


def resolve_ip_from_dns(urlorhost):
    # 解析IP地址，可传入IP,HOSTNAME,URL
    host = urlparse(urlorhost).hostname
    if not host:
        host = urlorhost
    try:
        ip = socket.gethostbyname(host)
    except socket.gaierror:
        ip = None

    return host, ip


def main(wf):
    # 去掉参数两边的空格
    param = (wf.args[0] if len(wf.args) else '').strip()

    if ":" in param:
        ip = extract_host(param)
        title = ip
    elif param:
        title, ip = resolve_ip_from_dns(param)
    else:
        title = get_local_ip()
        ip = get_public_ip()

    try:
        if ip:
            location_ipip = ipip(ip)
            render = u'{emoji}{ip} {result}'.format(
                emoji=location_ipip['emoji'] or '',
                ip=ip,
                result=location_ipip['result'] or ''
            )
            wf.add_item(title='IPIP.NET ' + title,
                        subtitle=render,
                        arg=render,
                        valid=True,
                        icon=location_ipip["flag"])

            if ':' not in param:
                location = get_location_information(ip)
                wf.add_item(title='IP138 ' + title,
                            subtitle=ip + ' ' + location if location else '',
                            arg=ip + ' ' + location if location else '',
                            valid=True,
                            icon=ICON_INFO)
        else:
            wf.add_item(title=title, subtitle='...', icon=ICON_ERROR)

    except Exception as e:
        wf.add_item(
            title=title,
            subtitle=u'IP地址不正确或域名解析出错，尝试IPv6则可能无法解析。',
            icon=ICON_ERROR
        )
        pass

    # Send the results to Alfred as XML
    wf.send_feedback()


if __name__ == u"__main__":
    wf = Workflow()
    sys.exit(wf.run(main))
